/*-
 * Copyright (c) 2008, Alexandre P. Francisco <aplf@ist.utl.pt>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _GCFND_H_
#define _GCFND_H_

/*
 * Edge data structure.
 */
typedef struct {
	int u;
	int v;
} edge;

/*-
 * Given an undirected graph, 'gcfdn' function computes the vertex
 * communities accordingly to the greedy approach of Clauset, Newman and
 * Moore. However, this is a different and faster implementation.
 *
 * Arguments:
 *
 *  nv   -   number of vertices;
 *  elst -   edge list, where each edge MUST exist only once and where
 *           vertices are numbered from 0 to nv - 1;
 *  ne   -   number of edges;
 *  cid  -   array to store the community id for each vertex;
 *  cnb  -   int address where to store the number of communities found;
 *  Q    -   double address where to store the modularity score for
 *           communities found.
 */
void	gcfnd(int nv, edge *elst, int ne, int *cid, int *cnb, double *Q);

/*
 * Adjacency list node definition. This data structure is specially
 * designed to provide cross reference between adjacency lists with
 * an access time of O(1).
 */
struct adj_node {
	int 	id;
	int	u;
	int	v;
	char	state;
#ifdef _RAND_CMP
	int	rand;
#endif
	struct adj_node	*u_nxt;
	struct adj_node *u_prv;
	struct adj_node *v_prv;
	struct adj_node *v_nxt;
	double	dQ;
};

#endif /* _GCFND_H_ */
